unit MainFrm;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.OleServer, Outlook_TLB, ComObj,
  Vcl.StdCtrls, Vcl.ComCtrls, Vcl.ToolWin,
  System.Actions, Vcl.ActnList, System.ImageList, Vcl.ImgList,
  System.NetEncoding,
  RVStyle, RVScroll, RichView, RVEdit, RVUni, RVTypes, RVGrHandler,
  RichViewActions;

type
  TfrmMain = class(TForm)
    RichViewEdit1: TRichViewEdit;
    RVStyle1: TRVStyle;
    btnSend: TButton;
    btnSave: TButton;
    btnPrint: TButton;
    cmbAccounts: TComboBox;
    Label1: TLabel;
    txtTo: TEdit;
    Label2: TLabel;
    ActionList1: TActionList;
    ImageList1: TImageList;
    rvActionFontBold1: TrvActionFontBold;
    rvActionFontItalic1: TrvActionFontItalic;
    rvActionFontUnderline1: TrvActionFontUnderline;
    rvActionInsertPicture1: TrvActionInsertPicture;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    txtSubject: TEdit;
    Label3: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure btnSendClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure btnPrintClick(Sender: TObject);
    procedure RichViewEdit1SaveImage2(Sender: TCustomRichView;
      Graphic: TGraphic; SaveFormat: TRVSaveFormat; const Path,
      ImagePrefix: TRVUnicodeString; var ImageSaveNo: Integer;
      var Location: TRVUnicodeString; var DoDefault: Boolean);
  private
    { Private declarations }
  public
    { Public declarations }
   App : OutlookApplication;
   function CreateMail: _MailItem;
   function GetSelectedAccount: _Account;
   function GetEmailBody: UnicodeString;
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}


procedure TfrmMain.FormCreate(Sender: TObject);
var
 Account: _Account;
 i: Integer;
begin
  try
    // Creating Outlook Application object
    App := CreateOleObject('Outlook.Application') as _Application;
    // List of accounts --> cmbAccounts
    for i := 1 to App.Session.Accounts.Count do
    begin
      Account := App.Session.Accounts.Item(i);
      // probably we should check Account.AccountType here
      cmbAccounts.Items.Add(Account.DisplayName);
    end;
    if cmbAccounts.Items.Count = 0 then
    begin
      Application.MessageBox('Outlook accounts are not found', nil, MB_OK or MB_ICONSTOP);
      Application.Terminate;
    end;
    cmbAccounts.ItemIndex := 0;
  except
    Application.MessageBox('Cannot initialize Outlook', nil, MB_OK or MB_ICONSTOP);
    Application.Terminate;
  end;
end;

// Returns e-mail body as HTML
function TfrmMain.GetEmailBody: UnicodeString;
var
  Stream: TMemoryStream;
begin
  Stream := TMemoryStream.Create;
  try
    RichViewEdit1.SaveHTMLToStreamEx(Stream, '', '', '', '', '', 'cp',
      [rvsoMiddleOnly, rvsoUTF8, rvsoUseCheckpointsNames, rvsoInlineCSS]);
    Stream.Position := 0;
    Result := RVU_AnsiStreamToUnicodeString(Stream, CP_ACP);
  finally
    Stream.Free;
  end;
end;

// Returns POP3 account selected in cmbAccounts
function TfrmMain.GetSelectedAccount: _Account;
var
  i: Integer;
begin
  for i := 1 to App.Session.Accounts.Count do
    begin
      Result := App.Session.Accounts.Item(i);
      if Result.DisplayName = cmbAccounts.Items[cmbAccounts.ItemIndex] then
       exit;
    end;
  Result := nil;
end;

// Creates MailItem and fills its properties
function TfrmMain.CreateMail: _MailItem;
begin
  Result := App.CreateItem(olMailItem) as _MailItem;
  Result.Subject := txtSubject.Text;
  // Result.To_ := txtTo.Text;
  Result.Recipients.Add(txtTo.Text);
  Result.Recipients.ResolveAll;
  Result.SendUsingAccount := GetSelectedAccount;
  Result.BodyFormat := olFormatHTML;
  Result.HTMLBody := GetEmailBody;
end;

// Printing e-mail
procedure TfrmMain.btnPrintClick(Sender: TObject);
begin
  CreateMail.PrintOut;
end;

// Saving e-mail
procedure TfrmMain.btnSaveClick(Sender: TObject);
begin
  CreateMail.Save;
  Application.MessageBox('The message was saved in Drafts', 'Info', MB_OK or MB_ICONINFORMATION);
end;

// Sending e-mail
procedure TfrmMain.btnSendClick(Sender: TObject);
begin
  CreateMail.Send;
  Application.MessageBox('The message was sent/placed to Outbox', 'Info', MB_OK or MB_ICONINFORMATION);
end;
{------------------------------------------------------------------------------}
// The code below is used to embed images directly in HTML
{------------------------------------------------------------------------------}
// If Gr is a graphic format appropriate for HTML, returns Gr
// Otherwise, returns Gr converted to Png
function GetHTMLGraphic(Gr: TGraphic): TGraphic;
var
  Bmp: TBitmap;
begin
  if RVGraphicHandler.IsHTMLGraphic(Gr) then
  begin
    Result := Gr;
    exit;
  end;
  Result := nil;
  try
    if RVGraphicHandler.GetGraphicType(Gr) = rvgtIcon then
      Result := RVGraphicHandler.IconToPng(Gr)
    else
    begin
      Result := RVGraphicHandler.CreateGraphicByType(rvgtPNG);
      try
        Result.Assign(Gr);
      except
        Bmp := RVGraphicHandler.ToBitmap(Gr);
        try
          Result.Assign(Bmp);
        finally
          Bmp.Free;
        end;
      end;
    end;
  except
    Result.Free;
    Result := Gr;
  end;
end;

function GetImageContentType(gr: TGraphic): UnicodeString;
begin
  case RVGraphicHandler.GetGraphicType(gr) of
    rvgtBitmap:
      Result := 'image/bmp';
    rvgtIcon:
      Result := 'image/vnd.microsoft.icon';
    rvgtJPEG:
      Result := 'image/jpeg';
    rvgtPNG:
      Result := 'image/png';
    rvgtGIF:
      Result := 'image/gif';
    rvgtTIFF:
      Result := 'image/tiff';
    else
      Result := 'application/octet-stream';
  end;
end;

procedure TfrmMain.RichViewEdit1SaveImage2(Sender: TCustomRichView;
  Graphic: TGraphic; SaveFormat: TRVSaveFormat; const Path,
  ImagePrefix: TRVUnicodeString; var ImageSaveNo: Integer;
  var Location: TRVUnicodeString; var DoDefault: Boolean);
var
  StreamIn, StreamOut: TMemoryStream;
  Base64Data: TRVAnsiString;
  GraphicToSave: TGraphic;
  i: Integer;
begin
  if SaveFormat <> rvsfHTML then
    exit;
  StreamIn := TMemoryStream.Create;
  StreamOut := TMemoryStream.Create;
  GraphicToSave := GetHTMLGraphic(Graphic);
  try
    GraphicToSave.SaveToStream(StreamIn);
    StreamIn.Position := 0;
    TNetEncoding.Base64.Encode(StreamIn, StreamOut);
    StreamOut.Position := 0;
    SetLength(Base64Data, StreamOut.Size);
    StreamOut.ReadBuffer(PRVAnsiChar(Base64Data)^, StreamOut.Size);
    for i := Length(Base64Data) downto 1 do
      if Base64Data[i] in [#13, #10] then
        Delete(Base64Data, i, 1);
    Location := 'data:' + GetImageContentType(GraphicToSave) + ';base64,' +
      String(Base64Data);
    DoDefault := False;
  finally
    StreamIn.Free;
    StreamOut.Free;
    if GraphicToSave <> Graphic then
      GraphicToSave.Free;
  end;
end;


end.
